package com.sd.framework.util.sql;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.support.LambdaMeta;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.sd.framework.util.sql.mapper.JoinMapper;
import org.apache.ibatis.reflection.property.PropertyNamer;

import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class Sql<T> {

    protected static final Map<Class<?>, String> tableNameMap = new ConcurrentHashMap<>();
    protected static final JoinMapper mapper = SpringUtil.getBean(JoinMapper.class);

    protected final Class<T> resultCls;
    protected final Map<Class<?>, String> tableAliasMap = new HashMap<>();
    protected final List<String> selectList = new ArrayList<>();
    protected final List<String> fromList = new ArrayList<>();

    protected Boolean distinct = false;

    protected Sql(Class<T> resultCls) {
        this.resultCls = resultCls;
    }

    public static <T> Select<T> create(Class<T> cls) {
        return new Select<>(new Sql<>(cls));
    }

    /**
     * 获取表名
     */
    protected static String getTableName(Class<?> cls) {
        return tableNameMap.computeIfAbsent(cls, itemCls -> {
            Annotation[] annotations = itemCls.getAnnotations();
            TableName tableName = null;
            for (Annotation annotation : annotations) {
                if (annotation instanceof TableName) {
                    tableName = (TableName) annotation;
                    break;
                }
            }
            String tableNameStr = null;
            if (tableName != null) {
                tableNameStr = tableName.value();
            }
            if (StrUtil.isBlank(tableNameStr)) {
                tableNameStr = StrUtil.toUnderlineCase(itemCls.getSimpleName());
            }
            return tableNameStr;
        });
    }

    protected String getTableAlias(Class<?> cls) {
        return tableAliasMap.computeIfAbsent(cls, itemCls -> "t" + (tableAliasMap.size() + 1));
    }

    protected String getTableAlias(SFunction<?, ?> function) {
        Class<?> cls = LambdaUtils.extract(function).getInstantiatedClass();
        return getTableAlias(cls);
    }

    protected <K, V> String getSqlFieldName(SFunction<K, V> function) {
        LambdaMeta meta = LambdaUtils.extract(function);
        String alias = getTableAlias(meta.getInstantiatedClass());
        String fieldName = StrUtil.toUnderlineCase(PropertyNamer.methodToProperty(meta.getImplMethodName()));
        return alias + StringPool.DOT + fieldName;
    }

    protected <K, V> String getSqlFieldName(String alias, SFunction<K, V> function) {
        LambdaMeta meta = LambdaUtils.extract(function);
        String fieldName = StrUtil.toUnderlineCase(PropertyNamer.methodToProperty(meta.getImplMethodName()));
        return alias + StringPool.DOT + fieldName;
    }

    protected String getSql() {
        return "SELECT" + (distinct ? "DISTINCT" : StringPool.EMPTY) + StringPool.SPACE +
                CollectionUtil.join(selectList, StringPool.COMMA) +
                CollectionUtil.join(fromList, StringPool.EMPTY);
    }

}
