package com.sd.wln.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuClassify;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.sku.SkuClassifyService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.wln.service.WlnSkuService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WlnSkuServiceImpl implements WlnSkuService {

    public static final Lock syncSkuLock = new ReentrantLock();

    @Autowired
    private SkuClassifyService skuClassifyService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sync() {
        if (syncSkuLock.tryLock()) {
            try {
                syncSkuClassify();
                syncSku();
            } finally {
                syncSkuLock.unlock();
            }
        }
    }

    /**
     * 同步sku分类
     */
    private void syncSkuClassify() {
        List<JSONObject> list;

        // 查询万里牛sku分类
        try {
            list = WlnUtil.getSkuClassifyList();
        } catch (Exception e) {
            log.error("sku分类同步失败", e);
            throw new ServiceException("sku分类同步失败");
        }

        // 查询数据库sku分类
        LogicHolder.setLogicHolder(true);
        Map<String, SkuClassify> skuClassifyMap = skuClassifyService.mapKEntity(SkuClassify::getWlnCatagoryId, null);
        LogicHolder.clear();

        // 去重合并
        List<SkuClassify> skuClassifyList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JSONObject item = list.get(i);

            String catagoryId = item.getString("catagoryid");
            String name = item.getString("catagory_name");
            String parentId = item.getString("parentid");

            SkuClassify skuClassify = skuClassifyMap.get(catagoryId);
            if (skuClassify == null) {
                skuClassify = new SkuClassify();
                skuClassify.setId(IdWorker.getId());
            }
            skuClassify.setWlnCatagoryId(catagoryId);
            skuClassify.setName(name);
            skuClassify.setWlnParentId(parentId);
            skuClassify.setSort(i);
            skuClassifyList.add(skuClassify);
        }

        // 赋值parentId
        Map<String, Long> map = skuClassifyList.stream()
                .collect(Collectors.toMap(SkuClassify::getWlnCatagoryId, SkuClassify::getId, (t1, t2) -> t1));
        for (SkuClassify skuClassify : skuClassifyList) {
            skuClassify.setParentId(map.getOrDefault(skuClassify.getWlnParentId(), 0L));
        }

        // 更新数据库
        skuClassifyService.saveOrUpdateBatch(skuClassifyList);
    }

    /**
     * 同步sku
     */
    private void syncSku() {

        // 获取万里牛sku列表
        List<JSONObject> wlnSkuList = getWlnSkuList();
        if (wlnSkuList.isEmpty()) {
            return;
        }

        List<String> wlnSkuIdList = wlnSkuList.stream().map(item -> item.getString("sys_goods_uid")).collect(Collectors.toList());

        LogicHolder.setLogicHolder(true);
        // 产品分类map
        Map<String, Long> skuClassifyMap = skuClassifyService.mapKV(SkuClassify::getWlnCatagoryId, BaseIdPo::getId, null);
        // 产品map
        Map<String, Sku> skuMap = skuService.mapKEntity(Sku::getWlnSkuId, q -> q.in(Sku::getWlnSkuId, wlnSkuIdList));
        // 产品规格map
        Map<Long, Map<String, SkuSpec>> specMap = skuSpecService.list(q -> q.in(SkuSpec::getWlnSkuId, wlnSkuIdList)).stream()
                .collect(Collectors.groupingBy(SkuSpec::getSkuId, Collectors.toMap(SkuSpec::getWlnSkuSpecId, Function.identity())));
        LogicHolder.clear();

        List<Sku> newSkuList = new ArrayList<>();
        List<SkuSpec> newSkuSpecList = new ArrayList<>();

        for (JSONObject wlnSku : wlnSkuList) {
            Sku tempSku = createSku(wlnSku, skuClassifyMap);
            if (tempSku == null) {
                continue;
            }
            Sku oldSku = skuMap.get(tempSku.getWlnSkuId());
            if (oldSku != null) {
                tempSku.setId(oldSku.getId());
            }
            newSkuList.add(tempSku);

            List<JSONObject> specList = wlnSku.getJSONArray("specs").toJavaList(JSONObject.class);
            for (JSONObject wlnSkuSpec : specList) {
                SkuSpec tempSkuSpec = createSkuSpec(wlnSkuSpec, tempSku);

                SkuSpec oldSkuSpec = null;
                Map<String, SkuSpec> oldSkuSpecMap = specMap.get(tempSku.getId());
                if (oldSkuSpecMap != null) {
                    oldSkuSpec = oldSkuSpecMap.get(tempSkuSpec.getWlnSkuSpecId());
                }
                if (oldSkuSpec != null) {
                    tempSkuSpec.setId(oldSkuSpec.getId());
                }

                newSkuSpecList.add(tempSkuSpec);
            }
        }

        skuService.saveOrUpdateBatch(newSkuList);
        skuSpecService.saveOrUpdateBatch(newSkuSpecList);
    }

    /**
     * 获取万里牛sku列表
     */
    private List<JSONObject> getWlnSkuList() {
        Sku lastSku = skuService.getOne(q -> q.orderByDesc(BasePo::getUpdateTime));

        Long modifyTime;
        if (lastSku == null) {
            modifyTime = 965381503L;
        } else {
            modifyTime = lastSku.getUpdateTime().getTime() + 1;
        }

        int page = 1;
        int size;
        List<JSONObject> wlnSkuList = new ArrayList<>();
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getSkuList(page, 200, modifyTime);
                wlnSkuList.addAll(itemList);
                page++;
                size = itemList.size();
            } catch (Exception e) {
                log.error("sku同步失败", e);
                throw new ServiceException("sku同步失败");
            }
        } while (size == 200);

        return wlnSkuList;
    }


    /**
     * 创建sku
     */
    private Sku createSku(JSONObject wlnSku, Map<String, Long> skuClassifyMap) {

        String catagoryId = wlnSku.getString("catagory_id");
        if (StrUtil.isBlank(catagoryId)) {
            return null;
        }

        Long skuClassifyId = skuClassifyMap.get(catagoryId);
        if (skuClassifyId == null) {
            return null;
        }

        Sku sku = new Sku();
        sku.setId(IdWorker.getId());
        sku.setWlnCatagoryId(catagoryId);
        sku.setWlnSkuId(wlnSku.getString("sys_goods_uid"));
        sku.setSource(2);
        sku.setSkuClassifyId(skuClassifyId);
        sku.setCode(wlnSku.getString("goods_code"));
        sku.setName(wlnSku.getString("goods_name"));
        sku.setBrand(wlnSku.getString("brand_name"));
        sku.setMainImgUrl(wlnSku.getString("pic"));
        sku.setType(Objects.equals(sku.getBrand(), "胜德科技") ? 1 : 0);
        sku.setRemark(wlnSku.getString("remark"));
        return sku;
    }

    /**
     * 创建sku规格
     */
    private SkuSpec createSkuSpec(JSONObject wlnSku, Sku sku) {
        SkuSpec skuSpec = new SkuSpec();
        skuSpec.setId(IdWorker.getId());
        skuSpec.setSkuId(sku.getId());
        skuSpec.setWlnSkuId(sku.getWlnSkuId());
        skuSpec.setWlnSkuSpecId(wlnSku.getString("sys_spec_uid"));
        skuSpec.setSpecImgUrl(wlnSku.getString("pic"));
        skuSpec.setCode(wlnSku.getString("spec_code"));
        skuSpec.setName(sku.getName() + ":" +
                ObjectUtil.defaultIfNull(wlnSku.getString("spec1"), StringPool.EMPTY) +
                ObjectUtil.defaultIfNull(wlnSku.getString("spec2"), StringPool.EMPTY));
        skuSpec.setBarCode(wlnSku.getString("barcode"));
        skuSpec.setLength(wlnSku.getBigDecimal("length"));
        skuSpec.setWidth(wlnSku.getBigDecimal("width"));
        skuSpec.setHeight(wlnSku.getBigDecimal("height"));
        skuSpec.setNetWeight(wlnSku.getBigDecimal("weight"));
        return skuSpec;
    }

}
